/* This program writes data to a specific block of a Mifare Classic tag
 *
 *
 * CAUTION: Be sure you understand the program in every detail,
 * since all data on your tag will be overwritten and can break
 * your tag, i.e. make the data unaccessible and unchangable.
 * USE AT YOUR OWN RISK!!!
 *
 * Modified by: Jan Scholtyssek <jan@janscholtyssek.dk>
 * Contains content of the example program mifareclassic_formatndef
 * from the Adafruit I2C Arduino library (see original description below).
 */

/**************************************************************************/
/*!
    @file     mifareclassic_formatndef.pde
    @author   Adafruit Industries
    @license  BSD (see license.txt)

    This example attempts to format a clean Mifare Classic 1K card as
    an NFC Forum tag (to store NDEF messages that can be read by any
    NFC enabled Android phone, etc.)

    Note that you need the baud rate to be 115200 because we need to print
    out the data and read from the card at the same time!

    This is an example sketch for the Adafruit PN532 NFC/RFID breakout boards
    This library works with the Adafruit NFC Shield
      ----> https://www.adafruit.com/products/789

    Check out the links above for our tutorials and wiring diagrams
    These chips use I2C to communicate

    Adafruit invests time and resources providing this open source code,
    please support Adafruit and open-source hardware by purchasing
    products from Adafruit!

*/
/**************************************************************************/

#include <Wire.h>
#include <Adafruit_NFCShield_I2C.h>

#define IRQ   (2)
#define RESET (3)  // Not connected by default on the NFC Shield

Adafruit_NFCShield_I2C nfc(IRQ, RESET);


void setup(void) {
  Serial.begin(115200);
  Serial.println("Looking for PN532...");

  nfc.begin();

  uint32_t versiondata = nfc.getFirmwareVersion();
  if (! versiondata) {
    Serial.print("Didn't find PN53x board");
    while (1); // halt
  }

  // Got ok data, print it out!
  Serial.print("Found chip PN5"); Serial.println((versiondata>>24) & 0xFF, HEX);
  Serial.print("Firmware ver. "); Serial.print((versiondata>>16) & 0xFF, DEC);
  Serial.print('.'); Serial.println((versiondata>>8) & 0xFF, DEC);

  // configure board to read RFID tags
  nfc.SAMConfig();
}

uint8_t blockNumber = 3;
uint8_t data[16] = { 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x78, 0x77, 0x88, 0x00, 0xA0, 0xA1, 0xA2, 0xA3, 0xA4, 0xA5 };
//uint8_t data[16] = { 0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF, 0xFE, 0xDC, 0xBA, 0x98, 0x76, 0x54, 0x32, 0x10 };
uint8_t readData[16];

void loop(void) {
  uint8_t success;                          // Flag to check if there was an error with the PN532
  uint8_t uid[] = { 0, 0, 0, 0, 0, 0, 0 };  // Buffer to store the returned UID
  uint8_t uidLength;                        // Length of the UID (4 or 7 bytes depending on ISO14443A card type)
  bool authenticated = false;               // Flag to indicate if the sector is authenticated

  // Use the default key
  uint8_t keya3[6] = { 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF };
  uint8_t keya2[6] = { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
  uint8_t keya1[6] = { 0xA0, 0xA1, 0xA2, 0xA3, 0xA4, 0xA5 };
  

  Serial.println("Place your Mifare Classic card on the reader and press any key to continue ...");
  // Wait for user input before proceeding
  while (!Serial.available());
  // a key was pressed1
  while (Serial.available()) Serial.read();

  // Wait for an ISO14443A type card (Mifare, etc.).  When one is found
  // 'uid' will be populated with the UID, and uidLength will indicate
  // if the uid is 4 bytes (Mifare Classic) or 7 bytes (Mifare Ultralight)
  success = nfc.readPassiveTargetID(PN532_MIFARE_ISO14443A, uid, &uidLength);

  if (success)
  {
    // Display some basic information about the card
    Serial.println("Found an ISO14443A card");
    Serial.print("  UID Length: ");Serial.print(uidLength, DEC);Serial.println(" bytes");
    Serial.print("  UID Value: ");
    nfc.PrintHex(uid, uidLength);
    Serial.println("");

    // Make sure this is a Mifare Classic card
    if (uidLength != 4)
    {
      Serial.println("Ooops ... this doesn't seem to be a Mifare Classic card!");
      return;
    }

    // We probably have a Mifare Classic card ...
    Serial.println("Seems to be a Mifare Classic card (4 byte UID)");
    Serial.println("Writing data:");
    nfc.PrintHexChar(data,16);
    
      Serial.print("Writing data to block "); Serial.println(blockNumber);
      success = nfc.mifareclassic_AuthenticateBlock (uid, uidLength, blockNumber, 1, keyb2);
      if (!success)
      {
        Serial.print("Unable to authenticate block "); Serial.print(blockNumber); Serial.println(" to enable card formatting!");
        return;
      }
      success = nfc.mifareclassic_WriteDataBlock(blockNumber, data);
      if (!success)
      {
        Serial.println("Unable to write data to card");
        return;
      }
      success = nfc.mifareclassic_AuthenticateBlock (uid, uidLength, blockNumber, 1, keyb);
      if (!success)
      {
        Serial.print("Unable to authenticate block "); Serial.print(blockNumber); Serial.println(" to read data!");
        return;
      }
      success = nfc.mifareclassic_ReadDataBlock(blockNumber, readData);
      if (!success)
      {
        Serial.println("Unable to read data from card");
        return;
      }
      nfc.PrintHexChar(readData,16);
    Serial.println("\nData successfully written");
  }

  // Wait a bit before trying again
  Serial.println("\nDone!");
  delay(1000);
  Serial.flush();
  while(Serial.available()) Serial.read();
}
